@@ -1,4 +1,5 @@
 from __future__ import unicode_literals, division, absolute_import
+import codecs
 import re
 import logging
 import os
@@ -145,17 +146,14 @@ class RegexpParse(object):
                 return False
         return entry.isvalid()
 
-    @cached('text')
+    @cached('regexp_parse')
     @plugin.internet(log)
     def on_task_input(self, task, config):
-
-        entries = []
-
         url = config['source']
 
-        #if it's a file open it and read into content
+        #if it's a file open it and read into content (assume utf-8 encoding)
         if os.path.isfile(os.path.expanduser(url)):
-            content = open(url).read()
+            content = codecs.open(url, 'r', encoding='utf-8').read()
         #else use requests to get the data
         else:
             content = task.requests.get(url).text
